/* HickCmd 1.1.0 -- Command line interface to hick.
** Copyright (C) 97, 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include <hickchk.h>

#include "hickcmd.h"

static void print_check_info(HickCheckEntry *head);


int
_check_packets(const char *packpath, const char *homepath,
	       const char *hicktype, unsigned int mask, time_t since)
{
  int err;
  HickCheck hchk;

  if((err = checkPackets(packpath, homepath, hicktype, mask, since, &hchk))) {
    return err;
  }

  switch(mask) {
    case CHECK_NEW: {
      if(!hchk.hc_new) {
	printf("No new packets or versions.\n");
      } else {
	printf("New packets and versions:\n");
	
	print_check_info(hchk.hc_new);
      }
      break;
    }
    case CHECK_CHANGED: {
      if(!hchk.hc_changed) {
	printf("No changes in your installed packets.\n");
      } else {
	printf("Changes in your installed packets:\n");

	print_check_info(hchk.hc_changed);
      }

      if(!hchk.hc_expire) {
	printf("No installed packets have expired or will expire.\n");
      } else {
	printf("Expire information:\n");

	print_check_info(hchk.hc_expire);
      }

      if(!hchk.hc_removed) {
	printf("No installed packets have been removed.\n");
      } else {
	printf("Removed packets or versions:\n");

	print_check_info(hchk.hc_removed);
      }
      break;
    }
  }
  
  return 0;
}


static void
print_check_info(HickCheckEntry *head)
{
  char buf[65];
  HickCheckEntry *chk;

  for(chk = head; chk; chk = chk->chk_next) {
    switch(chk->chk_flag) {
      case HICKCHK_NEWPACK: {	/* new packet */
	if(chk->chk_version[0]) {
	  printf("  packet '%s', default version is '%s'.\n", chk->chk_name,
		 chk->chk_version);
	} else {
	  printf("  packet '%s'.\n", chk->chk_name);
	}
	break;
      }
      case HICKCHK_NEWVER: {	/* new version */
	printf("  version '%s' in '%s'.\n", chk->chk_version, chk->chk_name);
	break;
      }
      case HICKCHK_CHDEF: {	/* changed default version */
	printf("  In '%s' has default version changed from '%s' to '%s'.\n",
	       chk->chk_name, chk->chk_version, chk->chk_nVersion);
	break;
      }
      case HICKCHK_MODPACK: {	/* modified packet */
	printf("  The packet '%s' have been modified.\n", chk->chk_name);
	break;
      }
      case HICKCHK_MODVER: {	/* modified version */
	printf("  The version '%s' in '%s' have been modified.\n",
	       chk->chk_version, chk->chk_name);
	break;
      }
      case HICKCHK_REMPACK: {	/* removed packet */
	printf("  Packets '%s' is removed.\n", chk->chk_name);
	break;
      }
      case HICKCHK_REMVER: {	/* removed version */
	if(chk->chk_nVersion[0]) {
	  printf("  Version '%s' in packet '%s' is removed. The default version is '%s'.\n", chk->chk_version, chk->chk_name, chk->chk_nVersion);
	} else {
	  printf("  Version '%s' in packet '%s' is removed.\n",
		 chk->chk_version, chk->chk_name);
	}
	break;
      }
      case HICKCHK_REMDEF: {	/* removed default version */
	printf("  Default version is no longer availible in '%s'.\n",
	       chk->chk_name);
	break;
      }
      case HICKCHK_EXPPACK: {	/* expired packet */
	printf("  The packet '%s' have expired.\n", chk->chk_name);
	break;
      }
      case HICKCHK_EXPVER: {	/* expired version */
	if(chk->chk_nVersion[0]) {
	  printf("  Version '%s' in '%s' have expire. The default version is '%s'.\n", chk->chk_version, chk->chk_name, chk->chk_nVersion);
	} else {
	  printf("  Version '%s' in '%s' have expired.\n", chk->chk_version,
		 chk->chk_name);
	}
	break;
      }
      case HICKCHK_ISEXPPACK: {	/* packet is going to expire */
	strftime(buf, 64, "%d/%m/%Y", localtime(&chk->chk_time));

	printf("  The packet '%s' will expire at %s.\n", chk->chk_name, buf);
	break;
      }
      case HICKCHK_ISEXPVER: {	/* version is going to expire */
	strftime(buf, 64, "%d/%m/%Y", localtime(&chk->chk_time));

	if(chk->chk_nVersion[0]) {
	  printf("  Versino '%s' in '%s' will expire at %s. The default version is '%s'.\n", chk->chk_version, chk->chk_name, buf, chk->chk_nVersion);
	} else {
	  printf("  Version '%s' in '%s' will expire at %s.\n",
		 chk->chk_version, chk->chk_name, buf);
	}
	break;
      }
    }
  }
}
