/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _AVAIL_H
#define _AVAIL_H

typedef struct _hup_apvinfo APVInfo;

#ifndef _HVIEW_H
typedef struct _hup_availpacket AvailPacket;
typedef struct _hup_availversion AvailVersion;
#endif /* _HVIEW_H */

#include "packet.h"
#include "project.h"
#include "gpixmap.h"
#include "specvars.h"

#define INSTALL_NOT     0
#define INSTALL_VERSION 1
#define INSTALL_DEFAULT 2
#define INSTALL_FORCE   3

#define CAN_NONE  0
#define CAN_ALL   1
#define CAN_SOME  2

/* APVInfo ***************************************************************** */

struct _hup_apvinfo {
  unsigned int rinstall : 2;	/* real install flag - for update */
  unsigned int install  : 2;	/* current install flag */
};

/* AvailPacket ************************************************************* */

struct _hup_availpacket {
  Node node;

  Packet *packet;		/* pointer to packet */

  PArr vers;			/* AvailVersion */
  PArr refs;			/* NTList ((RefPacket *) or (RefData *)) */

  APVInfo info;

  unsigned int can_def : 2;	/* can install default? */
};

/* AvailVersion ************************************************************ */

struct _hup_availversion {
  AvailPacket *packet;
  Version *version;

  APVInfo info;
};

#ifdef __cplusplus
extern "C" {
#endif

extern void avail_from_specvars(NodeList *packets, SpecVars *svars, 
				const char *instfile, NodeList *avail, 
				PArr *types, Project *prj);

extern void avail_from_specvars_edit(NodeList *packets, SpecVars *vars,
				     PArr *types, NodeList *avail);

extern void avail_to_specvars(NodeList *avail, PArr *types, SpecVars *svars, 
			      int setreal);

extern void avail_free(NodeList *avail);

extern int avail_set(AvailPacket *apack, AvailVersion *aver, int flag, 
		     int edit, NodeList *avail);

extern AvailPacket *avail_find_packet(const char *packet, NodeList *list);
extern AvailVersion *avail_find_version(const char *version, 
					AvailPacket *apack);

extern int avail_can_enable(AvailPacket *packet, AvailVersion *version, 
			    unsigned int install, int isedit, NodeList *avail);

#ifdef __cplusplus
}
#endif

#endif /* _AVAIL_H */
