/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _FILTER_H
#define _FILTER_H

#include "refs.h"

#if HAVE_REGEXEC || HAVE_REGEX
# define USE_REGEX
# ifdef HAVE_REGEXEC
#   include <regex.h>
# else /* not HAVE_REGEXEC */
#  include <libgen.h>
# endif /* not HAVE_REGEXEC */
#endif /* HAVE_REGEXEC || HAVE_REGEX */


#define FILTER_ALL     0

/* FilterMatch ************************************************************* */

typedef enum {
  FILTER_NONE          = 0,	/* no string match */
  FILTER_MATCH         = 1,	/* case insensitive match */
  FILTER_CASE_MATCH    = 3,	/* case sensitive match */

#ifdef USE_REGEX
  FILTER_REGEX_MATCH   = 5,	/* regex match */
#endif /* USE_REGEX */
} FilterMatch;

/* FilterSelect ************************************************************ */

typedef enum {
  FILTER_HIDDEN      = 0x10,	/* no hidden */
  FILTER_NOTSELECTED = 0x20,	/* no not selected  */
  FILTER_SELECTED    = 0x01,	/* no selected */
  FILTER_DEFAULT     = 0x02,	/* no defaults  */
  FILTER_FORCED      = 0x04,	/* no forced */
  FILTER_REQUIRED    = 0x08	/* no required */
} FilterSelect;

#define FILTER_SELECT_ALL      (FILTER_HIDDEN | FILTER_NOTSELECTED | \
				FILTER_SELECTED | FILTER_DEFAULT | \
				FILTER_FORCED | FILTER_REQUIRED) \

#define FILTER_SELECT_HIDE_NORMAL   (FILTER_HIDDEN)
#define FILTER_SELECT_SHOW_NORMAL   (0)

/* FilterTime ************************************************************** */

typedef enum {
  FILTER_CREATED  = 1,		/* check created <date> */
  FILTER_MODIFIED = 3,		/* check modified <date> */
  FILTER_EXPIRE   = 5,		/* check expire <date> */
} FilterTime;

/* Filter ****************************************************************** */

typedef struct {
  unsigned short only_ok;	 /* add only versions (and packets) which are
				 ** ok with the filter. */

  char *match_str;		 /* match string */
  char *match_strcase;		 /* match string lower case only */
  unsigned short match_not;	 /* true if it should not match? */
  unsigned short match_versions; /* true if we should match with versions? */
  unsigned short match;		 /* match mode */

#ifdef HAVE_REGEXEC
  regex_t preg;
  unsigned short preg_alloc;
#endif /* HAVE_REGEXEC */

  unsigned short select_hide;	 /* hide selection mask */
  unsigned short select_show;	 /* show selection mask */

  unsigned short time_show;	 /* time mode */
  unsigned short time_before;	 /* true if the time should be before spec? */
  unsigned short time_versions;	 /* true if we should check the versions? */
  time_t time_check;		 /* date to check with */
} Filter;

#ifdef __cplusplus
extern "C" {
#endif

extern void filter_init(Filter *filter);
extern void filter_clear(Filter *filter);
extern void filter_copy(Filter *to, Filter *from);

extern void filter_get(Filter *filter, HView *hview);

extern void filter_gui_set(Filter *filter, TabFilter *ftab);
extern void filter_gui_get(Filter *filter, TabFilter *ftab);

extern void cb_filter_reset(GtkWidget *w, gpointer data);
extern void cb_filter_newpackets(GtkWidget *w, gpointer data);
extern void cb_filter_update_date(GtkWidget *w, gpointer data);

#ifdef __cplusplus
}
#endif

#endif /* _FILTER_H */
