/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtknotebook.h>

#include "gui.h"


void
gui_main_init(GUI_Main *gmain, const char *prjsstr, char *user, char *homedir,
	      char *userdesc)
{
  gmain->window = NULL;
  gmain->factory = NULL;

  projects_init(&gmain->projects, prjsstr, user, homedir, userdesc);
  gui_pixmap_init(&gmain->pixmaps);

  dialog_init(&gmain->view_log);
  dialog_init(&gmain->search);
  dialog_msg_init(&gmain->about);
  dialog_err_init(&gmain->load_err_dialog);
}


Node *
gui_find_expand(const char *packet, const char *version, const char *type,
		Project *prj, int isedit)
{
  Node *node = NULL;
  GtkWidget *tree;

  if(type && 
     (node = node_find(type, (isedit ? hview_display_get(&prj->edit->hview) :
			      hview_display_get(&prj->packets.hview))))) {
    if(!packet && !version) {
      if(!GTK_CTREE_ROW(((NodeTree *)node)->ctnode)->expanded) {
	tree = (isedit ? prj->edit->hview.view : prj->packets.hview.view);

	gtk_clist_freeze(GTK_CLIST(tree));
	gtk_ctree_expand(GTK_CTREE(tree), ((NodeTree *)node)->ctnode);
	gtk_clist_thaw(GTK_CLIST(tree));
      }

      return node;
    }

    if((node = node_find_recursive(packet, NT_PACKET,
				   &((NTList *)node)->nlist))) {
      if(node && version) {
	node = node_find(version, &((NTList *)node)->nlist);
      }
    }
  }

  if(node) {
    gui_select_expand((NodeTree *)node, prj, isedit);
  }

  return node;
}


void
gui_select_expand(NodeTree *ntree, Project *prj, int isedit)
{
  size_t n;
  Node *node;
  GtkWidget *tree;

  tree = (isedit ? prj->edit->hview.view : prj->packets.hview.view);

  gtk_clist_freeze(GTK_CLIST(tree));

  if(prj != _gmain.projects.current) {	/* set project page */
    if(!(n = parr_find(&_gmain.projects.projects, prj, NULL))) {
      return;
    }

    _gmain.projects.current = prj;

    gtk_notebook_set_page(GTK_NOTEBOOK(_gmain.projects.notebook), (n - 1));
  }

  gtk_notebook_set_page(GTK_NOTEBOOK(_gmain.projects.current->notebook),
			(isedit ? 2 : 0));

  for(node = (Node *)ntree->parent; node; 
      node = (Node *)((NodeTree *)node)->parent) {
    if(!GTK_CTREE_ROW(((NodeTree *)node)->ctnode)->expanded) {
      gtk_ctree_expand(GTK_CTREE(tree), ((NodeTree *)node)->ctnode);
    }
  }

  if(ntree->node.flag != NT_VERSION) {
    gtk_ctree_expand(GTK_CTREE(tree), ntree->ctnode);
  }

  gtk_clist_thaw(GTK_CLIST(tree));
  gtk_ctree_select(GTK_CTREE(tree), ntree->ctnode);
  gtk_ctree_node_moveto(GTK_CTREE(tree), ntree->ctnode, 0, 0.5, 0.0);
}
