/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkmain.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtktext.h>
#include <gtk/gtkhbbox.h>
#include <gtk/gtklabel.h>
#include <gtk/gtktable.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkmenubar.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtktooltips.h>
#include <gtk/gtkscrolledwindow.h>

#include "gui.h"
#include "pref.h"
#include "load.h"
#include "wwrap.h"
#include "update.h"
#include "search.h"
#include "hickup.h"
#include "filter.h"
#include "buttons.h"
#include "specvars.h"
#include "availrep.h"


extern void cb_viewlog(GtkWidget *widget, gpointer data); /* viewlog.c */

static void create_project_book(Project *prj, GtkWidget *cont, 
				GUI_Pixmaps *pixmaps);

static void cb_errors(GtkWidget *w, gpointer data);
static void cb_run_update(GtkWidget *w, gpointer data);
static void cb_run_update_all(GtkWidget *w, gpointer data);
static void cb_load_report(GtkWidget *w, gpointer data);
static void cb_update_report(GtkWidget *w, gpointer data);
static void cb_about(GtkWidget *w, gpointer data);
static gint cb_quit_signal(gpointer data);

static void cb_switch_page(GtkWidget *w, GtkWidget *page, guint page_num, 
			   gpointer data);
static void cb_switch_project(GtkWidget *w, GtkWidget *page, guint page_num, 
			      gpointer data);


#define MENU_ENTRIES 20

static GtkItemFactoryEntry _menu_entries[MENU_ENTRIES] = {
  { "/_File",                  NULL,         NULL,          0, "<Branch>" },
  { "/File/_Reload","<control>r", 
    GTK_SIGNAL_FUNC(cb_reload_projects), 0, "<Item>" },
  { "/File/_Packet Errors...","<control>e", 
    GTK_SIGNAL_FUNC(cb_errors), 0, "<Item>" },
  { "/File/---",               NULL,         NULL,          0, "<Separator>" },
  { "/File/_Preferences...","<control>p", GTK_SIGNAL_FUNC(cb_pref),0,"<Item>"},
  { "/File/---",               NULL,         NULL,          0, "<Separator>" },
  { "/File/_Quit",       "<control>q", GTK_SIGNAL_FUNC(cb_quit), 0, "<Item>" },

  { "/_Find",                  NULL,         NULL,          0, "<Branch>" },
  { "/Find/_Find...",             "<control>f", 
    GTK_SIGNAL_FUNC(cb_find), 0, "<Item>" },
  { "/Find/Find _Again",       "<control>g", 
    GTK_SIGNAL_FUNC(cb_find_again), 0, "<Item>" },

  { "/_Packets",               NULL,         NULL,          0, "<Branch>" },
  { "/Packets/_View Log...","<control>v",
    GTK_SIGNAL_FUNC(cb_viewlog), 0, "<Item>" },
  { "/Packets/---",           NULL,         NULL,          0, "<Separator>" },
  { "/Packets/_Update", "<control>u",
    GTK_SIGNAL_FUNC(cb_run_update), 0, "<Item>" },
  { "/Packets/_Update All", "<control>l",
    GTK_SIGNAL_FUNC(cb_run_update_all), 0, "<Item>" },
  { "/Packets/---",           NULL,         NULL,          0, "<Separator>" },
  { "/Packets/L_oad Report...","<control>o",
    GTK_SIGNAL_FUNC(cb_load_report),0,"<Item>" },
  { "/Packets/Up_date Report...","<control>d",
    GTK_SIGNAL_FUNC(cb_update_report),0,"<Item>" },

  { "/_Help",                  NULL,         NULL,          0,"<LastBranch>" },
  { "/Help/_About...",         NULL,  GTK_SIGNAL_FUNC(cb_about), 0, "<Item>" }
};


GtkWidget *
gui_main_create(GUI_Main *gmain)
{
  size_t n;
  GtkAccelGroup *accel;
  char fname[FNAME_LEN+1];
  GtkWidget *vbox, *menubar, *workbox, *bbox, *pbox, *w;

  gtk_quit_add(0, cb_quit_signal, NULL);

  gmain->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(gmain->window, 620, 470);
  gtk_window_set_policy(GTK_WINDOW(gmain->window), TRUE, TRUE, FALSE);

  hsh_create_str(fname, FNAME_LEN, PROGRAM, " ", VERSION_STRING, 0);
  gtk_window_set_title(GTK_WINDOW(gmain->window), fname);

  gtk_signal_connect(GTK_OBJECT(gmain->window), "destroy",
		     GTK_SIGNAL_FUNC(gtk_widget_destroyed), &gmain->window);
  gtk_signal_connect(GTK_OBJECT(gmain->window), "delete_event",
		     GTK_SIGNAL_FUNC(cb_quit), NULL);

  gui_pixmap_create_all(&gmain->pixmaps, gmain->window);

  vbox = gtk_vbox_new(FALSE, 3);
  gtk_container_add(GTK_CONTAINER(gmain->window), vbox);

  accel = gtk_accel_group_new();
  gmain->factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<Main>", accel);
  gtk_item_factory_create_items(gmain->factory, MENU_ENTRIES, _menu_entries,
				NULL);
  gtk_window_add_accel_group(GTK_WINDOW(gmain->window), accel);
  menubar = gtk_item_factory_get_widget(gmain->factory, "<Main>");
  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);

  gtk_item_factory_parse_rc(gmain->menufile);
  gtk_widget_show(menubar);

  gmain->bsearch =
    gtk_item_factory_get_widget(GTK_ITEM_FACTORY(gmain->factory),
                                "/Find/Find Again");
  gtk_widget_set_sensitive(gmain->bsearch, FALSE);

  workbox = gtk_vbox_new(FALSE, 3);
  gtk_box_pack_start(GTK_BOX(vbox), workbox, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(workbox), 3);

  if(gmain->projects.projects.siz > 1) {
    gmain->projects.notebook = gtk_notebook_new();
    gtk_box_pack_start(GTK_BOX(workbox), gmain->projects.notebook,TRUE,TRUE,0);
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(gmain->projects.notebook), 
			     GTK_POS_BOTTOM);
    gtk_notebook_set_homogeneous_tabs(GTK_NOTEBOOK(gmain->projects.notebook), 
				      TRUE);
    gtk_notebook_set_tab_hborder(GTK_NOTEBOOK(gmain->projects.notebook), 5);
    gtk_notebook_set_show_border(GTK_NOTEBOOK(gmain->projects.notebook),FALSE);
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(gmain->projects.notebook), TRUE);

    gtk_signal_connect(GTK_OBJECT(gmain->projects.notebook), "switch_page",
		       GTK_SIGNAL_FUNC(cb_switch_project), &gmain->projects);

    for(n = 0; n < gmain->projects.projects.siz; n++) {
      pbox = gtk_vbox_new(FALSE, 0);
      w = gtk_label_new(parr_get(&gmain->projects.projects, Project, n)->name);

      gtk_notebook_append_page(GTK_NOTEBOOK(gmain->projects.notebook), pbox,
			       w);

      create_project_book(parr_get(&gmain->projects.projects, Project, n), 
			  pbox, &gmain->pixmaps);

      gtk_widget_show(pbox);
    }

    gtk_widget_show(gmain->projects.notebook);
  } else {
    create_project_book(gmain->projects.current, workbox, &gmain->pixmaps);
    cb_switch_project(NULL, NULL, 0, &gmain->projects);
  }

  gmain->sok = gmain->sapply = gmain->scancel = 0;
  bbox = ww_buttons(workbox, WW_HBBOX, GTK_BUTTONBOX_END, 3,
		    "OK",     BUTTON_ISDEF,  &gmain->bok,     NULL, NULL,
		    "Apply",  BUTTON_CANDEF, &gmain->bapply,  NULL, NULL,
		    "Cancel", BUTTON_CANDEF, &gmain->bcancel, NULL, NULL,
		    0);

  gtk_widget_show(workbox);
  gtk_widget_show(vbox);

  return gmain->window;
}


static void
create_project_book(Project *prj, GtkWidget *cont, GUI_Pixmaps *pixmaps)
{
  prj->notebook = gtk_notebook_new();
  gtk_box_pack_start(GTK_BOX(cont), prj->notebook, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(prj->notebook), GTK_POS_TOP);
  gtk_notebook_set_homogeneous_tabs(GTK_NOTEBOOK(prj->notebook), TRUE);
  gtk_notebook_set_tab_hborder(GTK_NOTEBOOK(prj->notebook), 5);
  
  create_packets_tab("Packets", &prj->packets, prj->notebook, pixmaps, prj);
  create_filter_tab(&prj->filter, prj->notebook);

  if(prj->can_edit) {
    prj->edit = MallocMem(TabPackets);
    create_edit_tab("Edit", prj->edit, prj->notebook, pixmaps, prj);
  }

  gtk_signal_connect(GTK_OBJECT(prj->notebook), "switch_page",
		     GTK_SIGNAL_FUNC(cb_switch_page), prj);

  gtk_widget_show(prj->notebook);
}


static gint
cb_quit_signal(gpointer date)
{
  gtk_item_factory_dump_rc(_gmain.menufile, NULL, 1);
  return 0;
}


static void
cb_switch_project(GtkWidget *w, GtkWidget *page, guint page_num, gpointer data)
{
  guint prj_page;
  Projects *prjs = (Projects *)data;

  prjs->current = parr_get(&prjs->projects, Project, page_num);

  if(prjs->current->packets.hview.view) {
    prj_page = 
      gtk_notebook_get_current_page(GTK_NOTEBOOK(prjs->current->notebook));

    buttons_signal(&_gmain, prj_page, prjs->current);

    if(!prjs->current->packets.has_init) {
      project_init_packets(prjs->current, &prjs->packets, &prjs->types);
    }
  }

  search_reset(-1);
}


static void
cb_switch_page(GtkWidget *w, GtkWidget *page, guint page_num, gpointer data)
{
  Project *prj = (Project *)data;

  buttons_signal(&_gmain, page_num, prj);

  switch(page_num) {
    case 0: {			/* packets tab */
      if(!prj->packets.has_init && prj->packets.hview.view) {
	project_init_packets(prj, &_gmain.projects.packets, 
			     &_gmain.projects.types);
      }
      break;
    }
    case 1: {			/* filter tab */
      filter_copy(&prj->filter.filter, &prj->packets.filter);
      filter_gui_set(&prj->filter.filter, &prj->filter);
      break;
    }
    case 2: {			/* edit tab */
      if(prj->edit && !prj->edit->has_init && prj->edit->hview.view) {
	project_init_edit(prj, &_gmain.projects.packets, 
			  &_gmain.projects.types);
      }
      break;
    }
  }

  search_reset(page_num);
}


static void 
cb_errors(GtkWidget *w, gpointer data)
{
  loaderr_do_show();
}


static void
cb_run_update(GtkWidget *w, gpointer data)
{
  cb_apply(w, _gmain.projects.current);
}


static void
cb_run_update_all(GtkWidget *w, gpointer data)
{
  update_all(&_gmain.projects, 1);
  cb_switch_project(0, 0, 0, &_gmain.projects);
}


static void
cb_load_report(GtkWidget *w, gpointer data)
{
  availrep_show(_gmain.projects.current);
}


static void
cb_update_report(GtkWidget *w, gpointer data)
{
  update_show(_gmain.projects.current);
}


static void
cb_about(GtkWidget *w, gpointer data)
{
  char title[128];

  if(!_gmain.about.dialog.window) {
    hsh_create_str(title, 127, "About ", PROGRAM, " ", VERSION_STRING, 0);

    dialog_msg_new(&_gmain.about, title, PROGRAM " " VERSION_STRING " - Hick "
		   "User Profiler\nby Daniel Bergstrm\n\nhttp://www.hickup."
		   "org/\n\nCredits To\nAnders Vidmark, Mattias Grnlund, "
		   "Ove Risberg and\nEricsson Erisoft AB", 
		   "OK", BUTTON_ISDEF, NULL, NULL, NULL, 0);
  }

  gtk_widget_show(_gmain.about.dialog.window);
}
