/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkmenu.h>
#include <gtk/gtkmenuitem.h>

#include "gui.h"
#include "menupop.h"


void
cb_select_clist(GtkWidget *clist, gint row, gint column, GdkEventButton *event,
		HView *hview)
{
  hview_select_set(hview,
		   (NodeTree *)gtk_clist_get_row_data(GTK_CLIST(clist), row));

}


void
cb_select_ctree(GtkWidget *ctree, GtkCTreeNode *row, gint column, 
		HView *hview)
{
  hview_select_set(hview, 
		   (NodeTree *)gtk_ctree_node_get_row_data(GTK_CTREE(ctree), 
							   row));
}


gint
cb_edit_key_item(GtkWidget *w, GdkEventKey *event, HView *hview)
{
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || event->type != GDK_KEY_PRESS || 
     !hview_selected_get_avail(hview, &apack, &aver) || (!apack && !aver)) {
    return FALSE;
  }

  if(event->keyval == ' ') {
    avail_set(apack, aver, -1, TRUE, hview->avail);  
    return TRUE;
  }

  return FALSE;
}


gint
cb_edit_click_item(GtkWidget *w, GdkEventButton *event, HView *hview)
{
  gint row, column;
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || (event->type != GDK_BUTTON_PRESS) || 
     !hview_select_get_avail(hview, event, &apack, &aver, &row, &column)) {
    return FALSE;
  }

  switch(event->button) {
    case 1: {			/* step install type by one */
      if(column || (!apack && !aver)) {
	return FALSE;
      }

      avail_set(apack, aver, -1, TRUE, hview->avail);
      return TRUE;
    }
    case 3: {			/* show popup menu */
      hview_select_row(hview, row);

      if(apack) {
	if(aver) {
	  hview_menu_set_gpixmap(hview, MENU_ENABLE, 
				 &_gmain.pixmaps.selected);
	  hview_menu_set_label(hview, MENU_ENABLE, "Enable Version");

	  hview_menu_set_show(hview, MENU_ENABLE_PACKET, TRUE);
	  hview_menu_set_show(hview, MENU_DISABLE_PACKET, TRUE);
	  hview_menu_set_show(hview, MENU_FORCE, TRUE);	  

	  hview_menu_set_sensitive(hview, MENU_ENABLE, 
		    avail_can_enable(apack, aver, INSTALL_VERSION, TRUE, 
				     hview->avail));
	  hview_menu_set_sensitive(hview, MENU_FORCE, 
		    avail_can_enable(apack, aver, INSTALL_FORCE, TRUE, 
				     hview->avail));
	} else {
	  hview_menu_set_gpixmap(hview, MENU_ENABLE,
				 &_gmain.pixmaps.dselected);
	  hview_menu_set_label(hview, MENU_ENABLE, "Enable Packet");
	  hview_menu_set_show(hview, MENU_ENABLE_PACKET, FALSE);
	  hview_menu_set_show(hview, MENU_DISABLE_PACKET, FALSE);
	  hview_menu_set_show(hview, MENU_FORCE, FALSE);

	  hview_menu_set_sensitive(hview, MENU_ENABLE, 
		    avail_can_enable(apack, NULL, INSTALL_VERSION, TRUE, 
				     hview->avail));
	}

	hview_menu_set_sensitive(hview, MENU_DEFAULT,  apack->can_def);
	hview_menu_set_sensitive(hview, MENU_DISABLE, TRUE);

	if(apack->packet->info.forced || (aver && aver->version->info.forced)){
	  hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
	}

	hview_menu_set_sensitive(hview, MENU_VIEW, TRUE);

	if(hview->menu.menu) {
	  gtk_menu_popup(GTK_MENU(hview->menu.menu), NULL, NULL, NULL, NULL, 
			 event->button, 0);
	}
      }
      
      return TRUE;
    }
  }

  return FALSE;
}


void 
cb_edit_view(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    gui_find_expand(hview->select.packet->packet->name, 
		    (!hview->select.version ? NULL :
		     hview->select.version->version->name),
		    hview->select.packet->packet->type,
		    hview->project, TRUE);
    
    
  }
}


void
cb_edit_enable(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, hview->select.version, 
	      (hview->select.version ? INSTALL_VERSION : INSTALL_DEFAULT), 
	      TRUE, hview->avail);
  }
}


void
cb_edit_enable_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_DEFAULT, TRUE, hview->avail);
  }
}


void
cb_edit_disable_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_NOT, TRUE, hview->avail);
  }
}


void
cb_edit_enable_default(GtkWidget *w, HView *hview)
{
  size_t n;
  AvailVersion *aver;

  if(!hview->select.packet || !hview->select.packet->can_def) {
    return;
  }

  for(n = 0; n < hview->select.packet->vers.siz; n++) {
    aver = parr_get(&hview->select.packet->vers, AvailVersion, n);

    avail_set(hview->select.packet, aver, 
	      (aver->version->isdef ? INSTALL_VERSION : INSTALL_NOT), 
	      TRUE, hview->avail);
  }
}


void
cb_edit_disable(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_NOT, TRUE, 
	      hview->avail);
  }
}


void
cb_edit_force(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_FORCE, 
	      TRUE, hview->avail);
  }
}


gint
cb_pack_key_item(GtkWidget *w, GdkEventKey *event, HView *hview)
{
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || event->type != GDK_KEY_PRESS || 
     !hview_selected_get_avail(hview, &apack, &aver) || (!apack && !aver)) {
    return FALSE;
  }

  if(event->keyval == ' ') {
    avail_set(apack, aver, -1, FALSE, hview->avail);  
    return TRUE;
  }

  return FALSE;
}


gint
cb_pack_click_item(GtkWidget *w, GdkEventButton *event, HView *hview)
{
  gint row, column;
  AvailVersion *aver;
  AvailPacket *apack, *apfpack;

  if(!event || (event->type != GDK_BUTTON_PRESS) || 
     !hview_select_get_avail(hview, event, &apack, &aver, &row, &column)) {
    return FALSE;
  }

  switch(event->button) {
    case 1: {                   /* step install type by one */
      if(column || (!apack && !aver)) {
        return FALSE;
      }

      avail_set(apack, aver, -1, FALSE, hview->avail);
      return TRUE;
    }
    case 3: {                   /* show popup menu */
      hview_select_row(hview, row);

      if(apack) {
	hview_menu_set_sensitive(hview, MENU_DEFAULT,  TRUE);
	hview_menu_set_sensitive(hview, MENU_DISABLE_PACKET,  TRUE);

	if(aver) {
	  hview_menu_set_show(hview, MENU_ENABLE, TRUE);   
	  hview_menu_set_show(hview, MENU_DISABLE, TRUE);
	} else {
	  hview_menu_set_show(hview, MENU_ENABLE, FALSE);   
	  hview_menu_set_show(hview, MENU_DISABLE, FALSE);
	}

	if(apack->packet->info.forced || (aver && aver->version->info.forced)){
	  hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
	} else {
	  if((apfpack= avail_find_packet(apack->packet->name, hview->avail))) {
	    if(apfpack->info.install == INSTALL_FORCE) {
	      hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
	    }
	  }
	}

	hview_menu_set_sensitive(hview, MENU_VIEW, TRUE);

	if(hview->menu.menu) {
	  gtk_menu_popup(GTK_MENU(hview->menu.menu), NULL, NULL, NULL, NULL, 
			 event->button, 0);
	}
      }

      return TRUE;
    }
  }

  return FALSE;
}


void 
cb_pack_view(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    gui_find_expand(hview->select.packet->packet->name, 
		    (!hview->select.version ? NULL :
		     hview->select.version->version->name),
		    hview->select.packet->packet->type,
		    hview->project, FALSE);
    
    
  }
}


void
cb_pack_select(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_VERSION, 
	      FALSE, hview->avail);
  }
}


void
cb_pack_select_default(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_DEFAULT, FALSE, 
	      hview->avail);
  }
}


void
cb_pack_unselect(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_NOT,
	      FALSE, hview->avail);
  }
}


void
cb_pack_unselect_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_NOT, FALSE, hview->avail);
  }
}
