/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkmain.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkframe.h>
#include <gtk/gtkdialog.h>

#include "gui.h"
#include "update.h"


typedef struct {
  PArr info;
  GtkWidget *clist;
} Rescue;


typedef struct {
  Project *project;
  char *type;
} RescueInfo;


static GUI_Dialog rdialog;
static Rescue rescue;

static void cb_cancel(GtkWidget *w, gpointer data);
static void cb_ok(GtkWidget *w, Rescue *rescue);

extern int check_rescue(GUI_Main *gmain);
extern void run_rescue(GUI_Main *gmain);


int
check_rescue(GUI_Main *gmain)
{
  char *name;
  size_t n, i;
  Project *prj;
  RescueInfo *rinfo;

  parr_init(&rescue.info);

  for(n = 0; n < gmain->projects.projects.siz; n++) {
    prj = parr_get(&gmain->projects.projects, Project, n);

    for(i = 0; i < gmain->projects.types.siz; i++) {
      name = parr_get(&gmain->projects.types, char, i);

      if(HickCheckRescue(prj->homedir, name)) {
	rinfo = MallocMem(RescueInfo);
	rinfo->project = prj;
	rinfo->type = name;
	parr_add(&rescue.info, rinfo);
      }
    }
  }

  if(rescue.info.siz == 0) {
    return 0;
  }

  return 1;
}


void
run_rescue(GUI_Main *gmain)
{
  size_t n;
  gchar *titles[2];
  RescueInfo *rinfo;
  GtkWidget *hbox, *frame, *box, *label;;

  dialog_init(&rdialog);

  dialog_new(&rdialog, "HickUP Rescue",
	     "OK", BUTTON_ISDEF, 0, cb_ok, &rescue,
	     "Cancel", BUTTON_CANDEF, 0, cb_cancel, 0,
	     0);

  gtk_signal_connect(GTK_OBJECT(rdialog.window), "delete_event",
                     GTK_SIGNAL_FUNC(cb_cancel), NULL);
  gtk_window_set_position(GTK_WINDOW(rdialog.window), GTK_WIN_POS_CENTER);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(rdialog.window)->vbox), hbox, FALSE, 
		     TRUE, 0);

  frame = gtk_frame_new(NULL);
  gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);

  box = gtk_hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(box), 5);
  gtk_container_add(GTK_CONTAINER(frame), box);

  label = gtk_label_new("Rescue is needed to be runned on the following "
			"OSes. Plese select those you wish to run rescue "
			"on. Rescue will reinstall all packets so that no "
			"conflicts may appear later on, while using HickUP.");
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_FILL);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, TRUE, 0);

  titles[0] = "Project";
  titles[1] = "OSes";

  rescue.clist =  gtk_clist_new_with_titles(2, titles);
  gtk_box_pack_start(GTK_BOX(hbox), rescue.clist, TRUE, TRUE, 0);
  gtk_clist_column_titles_passive(GTK_CLIST(rescue.clist));
  gtk_clist_set_column_min_width(GTK_CLIST(rescue.clist), 0, 120);
  gtk_clist_set_column_min_width(GTK_CLIST(rescue.clist), 1, 80);
  gtk_clist_set_column_auto_resize(GTK_CLIST(rescue.clist), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(rescue.clist), 1, TRUE);
  gtk_clist_set_selection_mode(GTK_CLIST(rescue.clist),GTK_SELECTION_MULTIPLE);

  for(n = 0; n < rescue.info.siz; n++) {
    rinfo = parr_get(&rescue.info, RescueInfo, n);
    titles[0] = rinfo->project->name;
    titles[1] = rinfo->type;
    gtk_clist_append(GTK_CLIST(rescue.clist), titles);
  }

  gtk_clist_select_all(GTK_CLIST(rescue.clist));
  gtk_widget_show_all(rdialog.window);
  gtk_main();
}


static void 
cb_cancel(GtkWidget *w, gpointer data)
{
  gtk_widget_hide(rdialog.window);
  gtk_main_quit();
}


static void 
cb_ok(GtkWidget *w, Rescue *r)
{
  gint row;
  GList *list;
  RescueInfo *rinfo;

  gtk_widget_hide(rdialog.window);

  if(GTK_CLIST(r->clist)->selection) {
    for(list = GTK_CLIST(r->clist)->selection; list; list = list->next) {
      row = GPOINTER_TO_INT(list->data);
      rinfo = parr_get(&r->info, RescueInfo, row);

      rescue_update(rinfo->project, rinfo->type);
    }
  }

  gtk_main_quit();
}
