/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <ctype.h>

#include <gtk/gtkbox.h>
#include <gtk/gtktable.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkcheckbutton.h>

#include "gui.h"
#include "search.h"
#include "hickup.h"

#define SEARCH_MAX_LEN   PNAME_LEN
#define SEARCH  ((Search *)_gmain.search.arg)

#define GET_START(chk, list) \
    (GTK_TOGGLE_BUTTON(chk)->active ? node_get_last(&(list)) : (list).head)

#define SELECTED_NODE(search) \
    ((search)->on_add == SEARCH_ADD ? &_gmain.add.ntree->node : \
     &_gmain.remove.ntree->node)

typedef struct {
  GtkWidget *text;
  GtkWidget *case_chk;		/* match case */
  GtkWidget *back_chk;		/* find backwards */
  GtkWidget *whole_chk;		/* match whole word only */
  GtkWidget *find_button;

  GUI_Dialog_Msg dialog;
  GtkWidget *dbutton_1;
  GtkWidget *dbutton_2;
  guint handler_id;

  Node *start;
  Node *current;
  HView *hview;

  char str[SEARCH_MAX_LEN + 1];

  unsigned short flag;
  unsigned short on_tab;	/* is it on which tab packets or edit? */
  unsigned short found;		/* have string been found at all? */
  unsigned short start_seen;	/* have we seen the start node? */
} Search;


static void cb_clear(GtkWidget *w, gpointer data);
static void cb_changed(GtkWidget *w, gpointer data);
static void cb_toggle(GtkWidget *w, gpointer data);
static void cb_search(GtkWidget *w, gpointer data);

static Node *node_get_last(NodeList *list);
static Node *node_get_next(Node *current, int back);

static int str_compare(const char *s1, const char *s2, Search *search);
static int str_case_compare(const char *s1, const char *s2, Search *search);
extern void str_ncasecpy(char *s1, const char *s2, size_t siz);

static void show_dialog(const char *msg, int buttons);
static void cb_start_over(GtkWidget *w, gpointer data);


void
cb_find(GtkWidget *w, gpointer data)
{
  Search *search;
  GtkWidget *window, *table, *ctable;

  if(!_gmain.search.window) {
    search = MallocMem(Search);
    search->found   = FALSE;
    search->start   = NULL;
    search->current = NULL;
    search->str[0]  = (char)0;
    search->handler_id = 0;
    search->dialog.dialog.window = NULL;

    window = dialog_new(&_gmain.search, "Find",
		"Find", BUTTON_ISDEF, &search->find_button, cb_search, search,
		"Clear", BUTTON_CANDEF, NULL, cb_clear, search,
		"Close", BUTTON_CANDEF, NULL, NULL, NULL, 0);

    SEARCH = search;
    gtk_widget_set_sensitive(search->find_button, FALSE);

    table = gtk_table_new(3, 2, FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(table), 10);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), table, TRUE,FALSE,0);

    w = gtk_label_new("Find What");
    gtk_table_attach(GTK_TABLE(table), w, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 5,5);

    search->text = gtk_entry_new_with_max_length(SEARCH_MAX_LEN);
    gtk_table_attach(GTK_TABLE(table), search->text, 1, 2, 0, 1,
		     ((GtkAttachOptions)(GTK_FILL | GTK_EXPAND | GTK_SHRINK)),
		     ((GtkAttachOptions)(GTK_FILL)), 0, 0);
    gtk_signal_connect(GTK_OBJECT(search->text), "activate",
		       GTK_SIGNAL_FUNC(cb_search), search);
    gtk_signal_connect(GTK_OBJECT(search->text), "changed",
		       GTK_SIGNAL_FUNC(cb_changed), search);

    ctable = gtk_table_new(2, 2, FALSE);
    gtk_table_attach(GTK_TABLE(table), ctable, 1, 2, 1, 2,
		     GTK_FILL, GTK_FILL, 0, 0);

    search->case_chk = gtk_check_button_new_with_label("Match Case");
    gtk_table_attach(GTK_TABLE(ctable), search->case_chk, 0, 1, 0, 1,
		     GTK_FILL, GTK_FILL, 0, 0);
    gtk_signal_connect(GTK_OBJECT(search->case_chk), "toggled",
		       GTK_SIGNAL_FUNC(cb_toggle), NULL);

    search->back_chk = gtk_check_button_new_with_label("Find Backwards");
    gtk_table_attach(GTK_TABLE(ctable), search->back_chk, 1, 2, 0, 1,
		     GTK_FILL, GTK_FILL, 0, 0);
    gtk_signal_connect(GTK_OBJECT(search->back_chk), "toggled",
		       GTK_SIGNAL_FUNC(cb_toggle), NULL);

    search->whole_chk =
      gtk_check_button_new_with_label("Match Whole Word Only");
    gtk_table_attach(GTK_TABLE(ctable), search->whole_chk, 0, 2, 2, 3,
		     GTK_FILL, GTK_FILL, 0, 0);
    gtk_signal_connect(GTK_OBJECT(search->whole_chk), "toggled",
		       GTK_SIGNAL_FUNC(cb_toggle), NULL);
  }

  gtk_widget_grab_focus(SEARCH->text);
  gtk_widget_show_all(_gmain.search.window);
}


void
search_reset(int page_num)
{
  Search *search = SEARCH;

  if(search) {
    if(page_num == -1) {
      search->on_tab = gtk_notebook_get_current_page(GTK_NOTEBOOK(
					   _gmain.projects.current->notebook));
    } else {
      search->on_tab = page_num;
    }

    if(search->on_tab == 1) {
      search->on_tab = 0;
    }

    search->hview = (search->on_tab ? &_gmain.projects.current->edit->hview : 
		     &_gmain.projects.current->packets.hview);

    if(search->hview->select.ntree) {
      search->current = &search->hview->select.ntree->node;
    } else {
      search->current = GET_START(search->back_chk, search->hview->display);
    }

    if(search->dialog.dialog.window) {
      gtk_widget_hide(search->dialog.dialog.window);
    }

    search->start = search->current;
    search->found = FALSE;
    search->start_seen = FALSE;
  }
}


static void
cb_toggle(GtkWidget *w, gpointer data)
{
  search_reset(-1);
  gtk_widget_grab_focus(SEARCH->text);
}


static void
cb_clear(GtkWidget *w, gpointer data)
{
  Search *search = (Search *)data;

  search->str[0] = (char)0;
  search->found  = FALSE;

  gtk_entry_set_text(GTK_ENTRY(search->text), "");
  gtk_widget_set_sensitive(_gmain.bsearch, FALSE);
  gtk_widget_grab_focus(SEARCH->text);
}


static void
cb_changed(GtkWidget *w, gpointer data)
{
  Search *search = (Search *)data;

  gtk_widget_set_sensitive(search->find_button,
			   (GTK_ENTRY(w)->text_length != 0));
}


static void
cb_search(GtkWidget *w, gpointer data)
{
  Search *search = (Search *)data;
  char *s = gtk_entry_get_text(GTK_ENTRY(search->text));

  if(!s || !*s) {
    gtk_widget_set_sensitive(_gmain.bsearch, FALSE);
    return;
  }

  if(!search->str[0] || strcmp(s, search->str)) {
    if(!*s) {
      search->str[0] = (char)0;
    } else {
      strncpy(search->str, s, SEARCH_MAX_LEN);
      search->str[SEARCH_MAX_LEN] = (char)0;
    }

    cb_toggle(NULL, NULL);
    gtk_widget_set_sensitive(_gmain.bsearch, TRUE);
  }

  cb_find_again(NULL, NULL);
}


void
cb_find_again(GtkWidget *w, gpointer data)
{
  Search *search = SEARCH;
  int (*cmpfunc)(const char *s1, const char *s2, Search *search);

  cmpfunc = (GTK_TOGGLE_BUTTON(search->case_chk)->active ? str_compare :
	     str_case_compare);

  do {
    while(search->current) {
      if(search->current == search->start) {
	if(search->start_seen && !search->found) {
	  show_dialog("Search string not found.", 1);
	  return;
	}

	search->start_seen = TRUE;
      }

      if(!(cmpfunc)(search->current->name, search->str, search)) {
	gui_select_expand((NodeTree *)search->current, 
			  _gmain.projects.current, (search->on_tab == 2));
	search->found   = TRUE;
	search->current =
	  node_get_next(search->current,
			GTK_TOGGLE_BUTTON(search->back_chk)->active);
	return;
      }

      search->current =
	node_get_next(search->current,
		      GTK_TOGGLE_BUTTON(search->back_chk)->active);
    }
  } while(search->current);

  if(GTK_TOGGLE_BUTTON(search->back_chk)->active) {
    show_dialog("Beginning of packets reached; continue from end?", 2);
  } else {
    show_dialog("End of packets reached; continue from beginning?", 2);
  }
}


static Node *
node_get_last(NodeList *list)
{
  Node *node;

  for(node = list->tail; node->flag != NT_VERSION; ) {
    node = ((NTList *)node)->nlist.tail;
  }

  return node;
}


static Node *
node_get_next(Node *current, int back)
{
  Node *node;

  if(!current) {
    return NULL;
  }

  if(back) {
    if(current->flag == NT_VERSION) {
      if(current->prev) {
	return current->prev;	/* move backward */
      }
    } else {
      if(current->prev) {
	return node_get_last(&((NTList *)current->prev)->nlist);
      }
    }

    return (Node *)((NodeTree *)current)->parent; /* move up */
  }

  if(current->flag == NT_VERSION) {
    if(current->next) {		/* move forward */
      return current->next;
    }
  } else {
    if(((NTList *)current)->nlist.head) { /* move deeper */
      return ((NTList *)current)->nlist.head;
    }
  }

  node = (Node *)((NodeTree *)current)->parent;	/* move up */

  while(node && !node->next) {
    node = (Node *)((NodeTree *)node)->parent;
  }

  return (node ? node->next : NULL);
}


static int
str_compare(const char *s1, const char *s2, Search *search)
{
  if(GTK_TOGGLE_BUTTON(search->whole_chk)->active) {
    return (strcmp(s1, s2));
  }

  return (strstr(s1, s2) == NULL);
}


static int
str_case_compare(const char *s1, const char *s2, Search *search)
{
  char str1[SEARCH_MAX_LEN+1], str2[SEARCH_MAX_LEN+1];

  if(GTK_TOGGLE_BUTTON(search->whole_chk)->active) {
    return strcasecmp(s1, s2);
  }

  str_ncasecpy(str1, s1, SEARCH_MAX_LEN);
  str1[SEARCH_MAX_LEN] = (char)0;

  str_ncasecpy(str2, s2, SEARCH_MAX_LEN);
  str2[SEARCH_MAX_LEN] = (char)0;

  return (strstr(str1, str2) == NULL);
}


void
str_ncasecpy(char *s1, const char *s2, size_t siz)
{
  size_t n;

  for(n = 0; (n < siz) && *s2; n++, s2++) {
    s1[n] = tolower(*s2);
  }

  if(n < siz) {
    s1[n] = (char)0;
  }
}


static void
show_dialog(const char *msg, int buttons)
{
  Search *search = SEARCH;

  if(!search->dialog.dialog.window) {
    dialog_msg_new(&search->dialog, PROGRAM ": Error", "",
		   "OK", BUTTON_ISDEF, &search->dbutton_1, cb_start_over, NULL,
		   "Cancel", BUTTON_CANDEF, &search->dbutton_2, NULL, NULL, 0);
  }

  gtk_widget_hide(search->dialog.dialog.window);

  gtk_label_set_text(GTK_LABEL(search->dialog.msg), msg);
  gtk_window_set_position(GTK_WINDOW(search->dialog.dialog.window),
			  GTK_WIN_POS_MOUSE);

  if(buttons == 2) {
    search->flag = TRUE;
    gtk_widget_show(search->dbutton_2);
  } else {
    search->flag = FALSE;
    gtk_widget_hide(search->dbutton_2);
  }

  gtk_widget_show(search->dialog.dialog.window);
}


static void
cb_start_over(GtkWidget *w, gpointer data)
{
  Search *search = SEARCH;

  gtk_widget_hide(search->dialog.dialog.window);

  if(search->flag) {
    if(GTK_TOGGLE_BUTTON(search->back_chk)->active) {
      search->current = node_get_last(&search->hview->display);
    } else {
      search->current = search->hview->display.head;
    }

    cb_find_again(NULL, NULL);
  }
}
