/* HickUP 1.1.1 -- Graphical interface to hick.
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <gtk/gtkhbox.h>
#include <gtk/gtktext.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkoptionmenu.h>

#include "common.h"		/* from hicklib 1.3 :: for LOG_FILE define */

#include "gui.h"
#include "pref.h"
#include "wwrap.h"

static void cb_change_type(GtkWidget *w, gpointer data);
static void cb_change_prj(GtkWidget *w, gpointer data);
static void show_log(size_t type, size_t prj);

extern void viewlog_update_hicktypes(void);
extern void cb_viewlog(GtkWidget *widget, gpointer data);

static size_t _type_pos = 0;
static size_t _prj_pos = 0;
static GtkWidget *_type_option = NULL;
static GtkWidget *_prj_option = NULL;


void
viewlog_update_hicktypes(void)
{
  GtkWidget *menu;

  if(_type_option) {
    menu = ww_menu((const char **)_gmain.projects.types.arr,
		   _gmain.projects.types.siz, GTK_SIGNAL_FUNC(cb_change_type));

    gtk_option_menu_set_menu(GTK_OPTION_MENU(_type_option), menu);
    gtk_option_menu_set_history(GTK_OPTION_MENU(_type_option), 0);
    _type_option = 0;
  }
}


void
cb_viewlog(GtkWidget *widget, gpointer data)
{
  GtkWidget *window, *hbox, *w;

  if(!_gmain.view_log.window) {
    window = dialog_new(&_gmain.view_log, "View Run Log",
			"OK", BUTTON_ISDEF, NULL, NULL, NULL, 0);
    gtk_widget_set_usize(window, 530, 300);
    gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), hbox, FALSE, TRUE,0);

    w = gtk_label_new("Log file for");
    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 0);

    _type_option = ww_array_option((const char **)_gmain.projects.types.arr,
			_gmain.projects.types.siz, 0, 
			GTK_SIGNAL_FUNC(cb_change_type));
    gtk_box_pack_start(GTK_BOX(hbox), _type_option, FALSE, TRUE, 0);

    if(_gmain.projects.projects.siz > 1) {
      w = gtk_label_new("in");
      gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 0);

      _prj_option = ww_array_option((const char **)_gmain.projects.names.arr,
				    _gmain.projects.names.siz, 0, 
				    GTK_SIGNAL_FUNC(cb_change_prj));
      gtk_box_pack_start(GTK_BOX(hbox), _prj_option, FALSE, TRUE, 0);
    }

    w = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), w, FALSE, FALSE, 0);

    w = ww_text_area((GtkWidget **)&_gmain.view_log.arg, 5);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), w, TRUE, TRUE, 0);
  }

  if(_prj_option) {
    _prj_pos = 
      gtk_notebook_get_current_page(GTK_NOTEBOOK(_gmain.projects.notebook));

    gtk_option_menu_set_history(GTK_OPTION_MENU(_prj_option), _prj_pos);
  }

  show_log(_type_pos, _prj_pos);
  gtk_widget_show_all(_gmain.view_log.window);
}


static void
cb_change_type(GtkWidget *w, gpointer data)
{
  show_log((_type_pos = (size_t)data), _prj_pos);
}


static void
cb_change_prj(GtkWidget *w, gpointer data)
{
  show_log(_type_pos, (_prj_pos = (size_t)data));
}


static void
show_log(size_t type, size_t prj)
{
  int fd;
  guint len;
  GtkText *text;
  char fname[FNAME_LEN];

  text = GTK_TEXT(_gmain.view_log.arg);
  gtk_text_freeze(text);

  if((len = gtk_text_get_length(text))) {
    gtk_text_backward_delete(text, len);
  }

  hsh_create_str(fname, FNAME_LEN,
		 parr_get(&_gmain.projects.projects, Project, prj)->homedir,
		 "/", _pref.hickdir, "/.",
		 _gmain.projects.types.arr[type], "/", LOG_FILE, 0);

  if((fd = open(fname, O_RDONLY, 0)) != -1) {
    while((len = read(fd, fname, FNAME_LEN)) > 0) {
      gtk_text_insert(text, NULL, NULL, NULL, fname, len);
    }

    close(fd);
  } else {
    gtk_text_insert(text, NULL, NULL, NULL, "Can't open runlog file '", -1);
    gtk_text_insert(text, NULL, NULL, NULL, fname, -1);
    gtk_text_insert(text, NULL, NULL, NULL, "'.", 2);
  }

  gtk_text_thaw(text);
}

 
