/* HMod 1.0.0 -- Hick modules emulation.
** Copyright (C) 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>

#include "hmod.h"
#include "memd.h"
#include "hstring.h"

static char *hickloaded_get(const char *str, size_t *pos, char endch);
static void hickloaded_convert(const char *str, HString *buf);


char *
hmod_strdup(const char *str)
{
  size_t len = strlen(str);
  char *p = MallocArr(char, strlen(str) + 1);

  hmod_out_of_memory(p);
  strcpy(p, str);
  return p;
}


void
hmod_out_of_memory(void *d)
{
  if(!d) {
    fprintf(stderr, "module: Out of memory -- exiting.\n");
    exit(1);
  }
}


int
hmod_hickloaded_get(const char *str, size_t *pos, char **name, char **version)
{
  if(!(*name = hickloaded_get(str, pos, '/'))) {
    return 0;
  }

  if(!(*version = hickloaded_get(str, pos, ':'))) {
    hmod_out_of_memory((*version = MallocArr(char, 1)));
    (*version)[0] = 0;
  }

  return 1;
}


char *
hmod_hickloaded_convert(char *name, char *version)
{
  HString buf;

  hstring_init(&buf);

  hickloaded_convert(name, &buf);
  hstring_add_char(&buf, '/');
  hickloaded_convert(version, &buf);
  hstring_eos(&buf);

  return hstring_data(&buf);
}


static void
hickloaded_convert(const char *str, HString *buf)
{
  size_t n;

  for(n = 0; str[n]; n++) {
    if((str[n] == ' ')) {
      hstring_add_char(buf, '\\');
    } else {
      if((str[n] == '\\') || (str[n] == '/') || (str[n] == ':') ||
	 (str[n] == ';')) {
	hstring_add_char(buf, '\\');
	hstring_add_char(buf, '\\');
      }
    }

    hstring_add_char(buf, str[n]);
  }
}


static char *
hickloaded_get(const char *str, size_t *pos, char endch)
{
  size_t n;
  HString res;

  if(*pos && !str[*pos - 1]) {
    return 0;
  }

  hstring_init(&res);

  for(n = *pos; str[n] && (str[n] != endch); n++) {
    if(str[n] == '\\') {
      switch(str[++n]) {
        case '\\':
        case '/':
        case ':': {
	  hstring_add_char(&res, str[n]);
	  continue;
	}
        default: {
	  hstring_add_char(&res, '\\');
	}
      }
    }
    
    hstring_add_char(&res, str[n]);
  }

  hstring_eos(&res);

  *pos = n + 1;
  return hstring_data(&res);
}
