/* HMod 1.0.0 -- Hick modules emulation.
** Copyright (C) 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>

#include "memd.h"

#include "hmod.h"
#include "hmodenv.h"
#include "hmodlist.h"

static int list_find(const void *d1, const void *d2);
static int list_find_pkg(const void *d1, const void *d2);
static int list_find_ver(const void *d1, const void *d2);


void
hmod_list_print(FILE *file)
{
  int n;
  PArr arr;
  HModLoaded *loaded;

  fprintf(file, "Currently loaded packets:\n");

  if(!hmod_list_get(&arr)) {
    return;
  }

  for(n = 0; n < arr.siz; n++) {
    loaded = parr_get(&arr, HModLoaded, n);

    fprintf(file, "%3d) %s/%s\n", n + 1, loaded->name, loaded->version);
  }
}


int
hmod_list_get(PArr *arr)
{
  size_t pos = 0;
  HModLoaded *lnew;
  char *env, *lname, *lver;

  parr_init(arr);

  if(!(env = getenv(HICKLOADED))) {
    return 0;
  }

  while(hmod_hickloaded_get(env, &pos, &lname, &lver)) {
    hmod_out_of_memory((lnew = MallocMem(HModLoaded)));
    lnew->name = lname;
    lnew->version = lver;

    parr_add(arr, lnew);
  }

  return 1;
}


void
hmod_list_add(PArr *arr, const char *name, const char *version)
{
  HModLoaded *lnew;

  if(hmod_list_find(arr, name, version)) {
    return;
  }

  hmod_out_of_memory((lnew = MallocMem(HModLoaded)));
  lnew->name    = hmod_strdup(name);
  lnew->version = hmod_strdup(version);
  parr_add(arr, lnew);
}


void
hmod_list_remove(PArr *arr, const char *name, const char *version)
{
  size_t n;
  HModLoaded fvalue;
  HModLoaded *lrem;

  fvalue.name = (char *)name;
  fvalue.version = (char *)version;

  if(!(n = parr_find(arr, &fvalue, list_find))) {
    return;
  }

  lrem = (HModLoaded *)parr_remove(arr, n - 1);
  FreeMem(lrem);
}


void
hmod_list_put(FILE *file, PArr *arr, const char *name)
{
  char *p;
  size_t n;
  HModEnv henv;
  HModEnvVar *hvar;

  hmod_env_init(&henv);
  hmod_out_of_memory((hvar = MallocMem(HModEnvVar)));

  hvar->name = hmod_strdup(HICKLOADED);
  hsh_nodelist_init(&hvar->values);
  hsh_node_prepend(&hvar->node, &henv.vars);

  for(n = 0; n < arr->siz; n++) {
    p = hmod_hickloaded_convert(parr_get(arr, HModLoaded, n)->name,
				parr_get(arr, HModLoaded, n)->version);

    hmod_env_value_append(hvar, p);
  }

  hmod_env_print(file, &henv, name);
}



HModLoaded *
hmod_list_find(PArr *arr, const char *name, const char *version)
{
  HModLoaded fvalue;

  fvalue.name = (char *)name;
  fvalue.version = (char *)version;

  return parr_find_value(arr, HModLoaded, &fvalue, list_find);
}


HModLoaded *
hmod_list_pkg_find(PArr *arr, const char *name)
{
  HModLoaded fvalue;

  fvalue.name = (char *)name;

  return parr_find_value(arr, HModLoaded, &fvalue, list_find_pkg);
}


HModLoaded *
hmod_list_ver_find(PArr *arr, const char *version)
{
  HModLoaded fvalue;

  fvalue.version = (char *)version;

  return parr_find_value(arr, HModLoaded, &fvalue, list_find_ver);
}


static int
list_find(const void *d1, const void *d2)
{
  return !(!strcasecmp(((HModLoaded *)d1)->name, ((HModLoaded *)d2)->name) &&
	   !strcasecmp(((HModLoaded *)d1)->version, 
		       ((HModLoaded *)d2)->version));
}

static int
list_find_pkg(const void *d1, const void *d2)
{
  return !(!strcasecmp(((HModLoaded *)d1)->name, ((HModLoaded *)d2)->name));
}


static int
list_find_ver(const void *d1, const void *d2)
{
  return !(!strcasecmp(((HModLoaded *)d1)->version,
		       ((HModLoaded *)d2)->version));
}
