/* HMod 1.0.0 -- Hick modules emulation.
** Copyright (C) 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "version.h"
#include "hmodrem.h"
#include "hmodlist.h"
#include "hmodload.h"

static void hmod_rem_environ(HickEnv *env, HModEnv *henv);

static int hmod_rem_parse_versions(HickPacket *hpkg, PArr *loaded, int tryadd,
				   PArr *vers, int isdef, HickVerList *nouse);


void
hmod_rem_get(HModEnv *henv, HickPackets *pkg, PArr *loaded, const char *shell,
	     char **packets, size_t siz)
{
  HModLoadUpdate update;

  update.func_ver       = hmod_rem_parse_versions;
  update.func_insted    = 0;
  update.func_conflict  = 0;
  update.func_env       = hmod_rem_environ;
  update.func_loaded    = hmod_list_remove;
  update.conflict_chk   = 0;
  update.environ_rescue = 1;

  hmod_load_update(&update, henv, pkg, loaded, 0, packets, siz);
}


static void
hmod_rem_environ(HickEnv *env, HModEnv *henv)
{
  if(env->env_flags != HFLAG_ENV_UNSET) {
    hmod_env_remove(henv, env->env_name, env->env_value);
  }
}


static int
hmod_rem_parse_versions(HickPacket *hpkg, PArr *loaded, int tryadd,
			PArr *vers, int isdef, HickVerList *nouse)
{
  HickVer *hver, *rver;

  if(!isdef) {			/* get matching version */
    for(hver = hpkg->pac_versions.vl_head; hver; ) {
      if(!hmod_list_ver_find(vers, hver->ver_number)) {
	rver = hver;
	hver = hver->ver_next;
	_versionMove(rver, &hpkg->pac_versions, nouse);
	continue;
      }

      if(!hmod_list_find(loaded, hpkg->pac_name, hver->ver_number)) {
	fprintf(stderr, "module: packet '%s/%s' is not loaded.\n",
		hpkg->pac_name, hver->ver_number);
	rver = hver;
	hver = hver->ver_next;
	_versionMove(rver, &hpkg->pac_versions, nouse);
	continue;
      }

      hver = hver->ver_next;
    }

    if(hpkg->pac_versions.vl_head) {
      return 1;
    }

    return 0;
  }

  for(hver = hpkg->pac_versions.vl_head; hver; ) {
    if(!hmod_list_find(loaded, hpkg->pac_name, hver->ver_number)) {
      rver = hver;
      hver = hver->ver_next;
      _versionMove(rver, &hpkg->pac_versions, nouse);
      continue;
    }
    
    hver = hver->ver_next;
  }

  if(hpkg->pac_versions.vl_head) {
    return 1;
  }

  return 1;
}
