/* HMod 1.0.0 -- Hick modules emulation.
** Copyright (C) 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include "hmodshell.h"


static HModShell shell_info[] = {
  {				/* SHELL_TYPE_SH */
    "%s=%s\nexport %s;\n",
    "unset %s;\n"
  },
  {				/* SHELL_TYPE_KSH */
    "export %s=%s;\n",
    "unset %s;\n"
  },
  {				/* SHELL_TYPE_CSH */
    "setenv %s %s;\n",
    "unsetenv %s;\n"
  },
  {				/* SHELL_TYPE_TCLSH */
    "set env(%s) %s;\n",
    "if [info exists env(%s)] {\n  unset env(%s)\n};\n"
  },
  {				/* SHELL_TYPE_ES */
    "%s=%s;\n",
    "%s=();\n"
  },
  {				/* SHELL_TYPE_RC */
    "%s=%s;\n",
    "%s=();\n"
  }
};


typedef struct {
  const char *name;
  HModShellType type;
} HModShells;


static HModShells shells[] = {
  { "sh",    SHELL_TYPE_SH },
  { "zsh",   SHELL_TYPE_SH },
  { "bash",  SHELL_TYPE_SH },
  { "ksh",   SHELL_TYPE_KSH },
  { "csh",   SHELL_TYPE_CSH },
  { "tcsh",  SHELL_TYPE_CSH },
  { "tclsh", SHELL_TYPE_TCLSH },
  { "es",    SHELL_TYPE_ES },
  { "rc",    SHELL_TYPE_RC },
  { 0,       SHELL_TYPE_NONE }
};


HModShellType
hmod_shell_type_get(const char *name)
{
  int n;

  for(n = 0; shells[n].name; n++) {
    if(!strcasecmp(shells[n].name, name)) {
      return shells[n].type;
    }
  }

  return SHELL_TYPE_NONE;
}


HModShell *
hmod_shell_get(HModShellType type)
{
  return (type == SHELL_TYPE_NONE ? 0 : &shell_info[type]);
}
