/* HSplit 1.0.1 -- Packet splitter
** Copyright (C) 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include "hick.h"

#define PROGRAM_NAME     "hsplit"
#define PROGRAM_VERSION  "1.0.1"

static int
usage(const char *filename)
{
  fprintf(stderr, "Usage: %s [ OPTIONS ] PACKET_PATH HICKTYPE PACKET "
	  "[ VERSION ... ]\n\n"
          "\t-h\t\tThis help text.\n"
          "\t-v\t\tVersion information.\n\n", filename);
  return 0;
}


int
main(int argc, char **argv)
{
  int n, defrem = 0;
  HickPacket packet;
  HickVer *ver, *last;
  char *name = argv[3];
  char *hicktype = argv[2];
  char *packet_path = argv[1];

  while((n = getopt(argc, argv, "hvd:")) != EOF) {
    switch(n) {
      case 'v': {               /* version information */
        printf(PROGRAM_NAME " " PROGRAM_VERSION "\n");
        return 0;
      }
      case 'h':                 /* usage info */
      default: {                /* ERROR! */
        return usage(argv[0]);
      }
    }
  }

  if(argc < 4) {
    return usage(argv[0]);
  }

  initHickPacket(&packet);

  switch(loadPacket(name, packet_path, hicktype, NULL, &packet, 1)) {
    case HICKERR_PACKDIR: {
      fprintf(stderr, "Failed to open packet directory '%s'.\n", packet_path);
      return 0;
    }
  }

  if(argc >= 5) {
    for(ver = packet.pac_versions.vl_head; ver; ) {
      for(n = 4; n < argc; n++) {
	if(!strcasecmp(ver->ver_number, argv[n])) {
	  break;
	}
      }

      if(n >= argc) {
	if(ver->ver_default) {
	  defrem = 1;
	}

	last = ver->ver_next;
	_versionRemove(ver, &packet.pac_versions);
	ver = last;
	continue;
      }

      ver = ver->ver_next;
    }
  }

  if(packet.pac_versions.vl_head && (packet.pac_name[0] != (char)0)) {
    if(defrem && !packet.pac_multiple) {
      packet.pac_versions.vl_head->ver_default = 1;
    }

    savePacket(stdout, &packet, hicktype, HTIME_DATE);
  }

  return 0;
}
