/* HICK Lib 1.3.2 - User Profiler Library.
** Copyright (C) 97, 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>

#ifdef STDC_HEADERS
# include <string.h>
#endif

#ifdef TM_IN_SYS_TIME
# include <sys/time.h>
#endif

#include "getfile.h"

#define NUM_LEN 9		/* Max length of a number */

#define MIN_TIM_LEN 10		/* Min length for a date. */
#define MID_TIM_LEN 16		/* Middle length, date with hours. */
#define MAX_TIM_LEN 19		/* Max length, with seconds. */


char *
_getName(FILE *file, char *buf, size_t maxlen)
{
  int c;
  size_t siz;

  if((c = _getSpaces(file)) == EOF) {
    return NULL;
  }

  for(siz = 0 ; (c != EOF) && !isspace(c) && (siz < maxlen); c = fgetc(file)) {
    buf[siz++] = c;
  }

  buf[siz] = 0;
  return buf;
}


char *
_getString(FILE *file)
{
  int c;
  char *buf = 0;
  size_t siz = 0, asiz = 0;

  if((c = _getSpaces(file)) == EOF) {
    return NULL;
  }

  asiz = REALLOC_SIZE;
  buf = MallocArr(char, asiz);

  if(c != '"') {
    for( ; (c != EOF) && !isspace(c); c = fgetc(file)) {
      _addChar(buf, siz, asiz, c);
    }
  } else {
    while(((c = fgetc(file)) != EOF) && (c != '"')) {
      if(c == '\\') {
	if((c = fgetc(file)) == EOF) {
	  break;
	}

	switch(c) {
	  case '"':
	  case '\\': {
	    _addChar(buf, siz, asiz, c);
	    continue;
	  }
	  case 'n': {
	    _addChar(buf, siz, asiz, '\n');
	    continue;
	  }
	  default: {
	    _addChar(buf, siz, asiz, '\\');
	  }
	}
      }

      _addChar(buf, siz, asiz, c);
    }
  }

  _addChar(buf, siz, asiz, 0);
  return buf;
}


long
_getNumber(FILE *file)
{
  int c;
  size_t n;
  char buf[NUM_LEN+1];

  if(((c = _getSpaces(file)) == (char)EOF) || !isdigit(c)) {
    return 0;
  }

  for(n = 0; (c != (char)EOF) && (n < NUM_LEN) && isdigit(c); c = fgetc(file)){
    buf[n++] = c;
  }

  buf[n] = (char)0;

  return atol(buf);
}


time_t
_getTime(FILE *file)
{
  int c, n, len;
  struct tm tmp;
  char buf[MAX_TIM_LEN+1], tmps[5];

  if((c = _getSpaces(file)) == EOF) { /* Read the date */
    return 0;
  }

  buf[(n = 0)] = c;

  for(n++, c = fgetc(file); (c != EOF) && !isspace(c); c = fgetc(file)) {
    buf[n++] = c;
  }

  buf[n] = (char)0;
  len = n;

  /* Check if they are all digits. */

  for(n = 0; (n < MAX_TIM_LEN) && isdigit((int)buf[n]); n++);

  if(buf[n] == (char)0) {	/* All digits? return that then. */
    return atol(buf);
  }

  /* To short date information? fail. */

  if((len < MIN_TIM_LEN) || (buf[2] != '/') || (buf[5] != '/')) {
    return -1;
  }

  tmp.tm_isdst = -1;
  tmp.tm_hour = tmp.tm_min = tmp.tm_sec = 0;

  strncpy(tmps, buf, 2);
  tmps[2] = (char)0;
  tmp.tm_mday = atoi(tmps);	/* Days*/

  strncpy(tmps, buf+3, 2);
  tmps[2] = (char)0;
  tmp.tm_mon = atoi(tmps) - 1;	/* Month */

  strncpy(tmps, buf+6, 4);
  tmps[4] = (char)0;
  tmp.tm_year = atoi(tmps) - 1900; /* Year */

  /* Check if we got any hours and minutes */

  if((len >= MID_TIM_LEN) && (buf[10] == ',') && (buf[13] == ':')) {
    strncpy(tmps, buf+11, 2);
    tmps[2] = (char)0;
    tmp.tm_hour = atoi(tmps);	/* Hours */

    strncpy(tmps, buf+14, 2);
    tmps[2] = (char)0;
    tmp.tm_min = atoi(tmps);	/* Minutes */

    /* Check if we got any seconds */

    if((len >= MAX_TIM_LEN) && (buf[16] == '.')) {
      strncpy(tmps, buf+17, 2);
      tmps[2] = 0;
      tmp.tm_sec = atoi(tmps);	/* Seconds */
    }
  }

  return mktime(&tmp);
}


void
_getComment(FILE *file)
{
  int c;

  while(((c = fgetc(file)) != (char)EOF) && (c != '\n'));
}


int
_getSpaces(FILE *file)
{
  int c;

  while(((c = fgetc(file)) != EOF) && isspace(c));

  return c;
}


void
_readStmt(FILE *file, int start)
{
  int c;

  if(start && ((c = _getSpaces(file)) != '{')) {
    ungetc(c, file);		/* put back what ever we got. */
    return;
  }

  while(((c = fgetc(file)) != (char)EOF) && (c != '}')) {
    switch(c) {
      case '{': {		/* Read statement */
	_readStmt(file, 0);
	break;
      }
      case '"': {		/* Read string */
	_readString(file, 0);
	break;
      }
      case '\n': {		/* New line, possible comment */
	c = _getSpaces(file);

	if(c == '#') {
	  _getComment(file);
	} else {
	  if((c == '}') || (c == '{') || (c == '"')) {
	    ungetc(c, file);
	  }
	}
	break;
      }
    }
  }
}


void
_readString(FILE *file, int start)
{
  int c;

  if(start && (_getSpaces(file) != '"')) {
    while(((c = fgetc(file)) != EOF) && !isspace(c));
  } else {
    for(c = _getSpaces(file); (c != EOF) && (c != '"'); c = fgetc(file)) {
      if((c == '\\') && ((c = fgetc(file)) == EOF)) {
	break;
      }
    }
  }
}
