/* HICK Lib 1.3.2 - User Profiler Library.
** Copyright (C) 97, 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "common.h"

int
getHickTypes(const char *path, ArrayInfo *result)
{
  uid_t uid;
  gid_t gid;
  DIR *dirp;
  struct stat sbuf;
  struct dirent *entry;
  char fname[FNAME_LEN+1];

  if(!(dirp = hsh_opendir(path, NULL))) {
    return HICKERR_TYPEDIR;
  }

  uid = geteuid();
  gid = getegid();

  hsh_array_init(result);

  while((entry = readdir(dirp))) {
    if(IS_DOT_OR_DOTDOT(entry) || !strcasecmp(entry->d_name, COMMON_DIR)) {
      continue;
    }

    hsh_create_str(fname, FNAME_LEN, path, "/", entry->d_name, 0);

    if((hsh_stat(fname, &sbuf, NULL) == IOW_FAIL) || !S_ISDIR(sbuf.st_mode)) {
      continue;
    }

    if(((sbuf.st_uid == uid) && (sbuf.st_mode & (S_IRUSR | S_IXUSR))) || 
       ((sbuf.st_gid == gid) && (sbuf.st_mode & (S_IRGRP | S_IXGRP))) ||
       ((sbuf.st_mode & (S_IROTH | S_IXOTH)))) {

      if(!hsh_array_add(entry->d_name, 1, result)) {
	hsh_array_remove(result);
	return HICKERR_OUTOFMEMORY;
      }
    }
  }

  closedir(dirp);
  return 0;
}
