/* HICK Lib 1.3.2 - User Profiler Library.
** Copyright (C) 97, 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _HICKCHK_H
#define _HICKCHK_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>

#ifndef HICKLIB_MAJOR_VERSION
#define HICKLIB_MAJOR_VERSION 1
#endif

#ifndef HICKLIB_MINOR_VERSION
#define HICKLIB_MINOR_VERSION 3
#endif

#ifndef PVER_LEN
#define PVER_LEN   16		/* max verison length of a packet */
#endif

#ifndef PNAME_LEN
#define PNAME_LEN  32		/* max name length of a packet */
#endif

#define CHK_NEW      (1<<1)	/* Check for new packets */
#define CHK_CHANGED  (1<<2)	/* Check for changed packet  information */
#define CHK_REMOVED  (1<<3)	/* Notify about removed installed packets */
#define CHK_EXPIRE   (1<<4)	/* Give expire information */

enum hickCheck {
  HICKCHK_NEWPACK,		/* New packet */
  HICKCHK_NEWVER,		/* New version */

  HICKCHK_CHDEF,		/* Default version changed */
  HICKCHK_MODPACK,		/* Modified packet */
  HICKCHK_MODVER,		/* Modified version */

  HICKCHK_REMPACK,		/* Removed packet */
  HICKCHK_REMVER,		/* Removed version */
  HICKCHK_REMDEF,		/* Default version removed */

  HICKCHK_EXPPACK,		/* Expired packet */
  HICKCHK_EXPVER,		/* Expired version */
  HICKCHK_ISEXPPACK,		/* Packet is expiering */
  HICKCHK_ISEXPVER		/* Version is expiering */
};

/* HickCheckEntry ********************************************************** */

typedef struct _hchkentry {
  struct _hchkentry *chk_next;	/* pointer to next HickCheck in list. */

  enum hickCheck chk_flag;	/* flag descriping the information. */

  char chk_name[PNAME_LEN+1];	 /* packet name  */
  char chk_version[PNAME_LEN+1]; /* version number */
  char chk_nVersion[PVER_LEN+1]; /* new or default version number */

  time_t chk_time;		/* expired, changed, created dates */
} HickCheckEntry;

/* New Packet          : default (chk_version)
** New Version	       : new version (chk_version)
** Changed Default     : old version (chk_version) new version (chk_nVersion)
** Modified Version    : version (chk_version)
** Removed Version     : version (chk_version), default (chk_nVersion)
** Removed Packet      : old version (chk_version)
** Expired Version     : version (chk_version), default (chk_nVersion)
** Will Expire Version : version (chk_version), default (chk_nVersion)
**
** the default versions is depending on if there is any.
*/

/* HickCheck *************************************************************** */

typedef struct {
  HickCheckEntry *hc_new;	/* New packets. */
  HickCheckEntry *hc_changed;	/* Changed packets / version. */
  HickCheckEntry *hc_removed;	/* Removed packets / versions. */
  HickCheckEntry *hc_expire;	/* Expired packets / versions or to expire. */
} HickCheck;

/* Extern functions ******************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

extern int checkPackets(const char *packpath, const char *homepath,
			const char *hicktype, unsigned int mask,
			time_t newsince, HickCheck *result);

extern HickCheck *freeHickCheck(HickCheck *hc);

#ifdef __cplusplus
}
#endif

#endif /* _HICKCHK_H */
