/* HICKMK Lib 1.2.1 - HICK Make Utility
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _MKINTERP_H		/* Interpreter for MK Language */
#define _MKINTERP_H

#include <stdio.h>

typedef struct _mk_interpreter MKInterp;

#include "sbuf.h"
#include "mklang.h"
#include "mkparse.h"

/* MKVariable ************************************************************** */

typedef struct _mk_variable MKVariable;

typedef int (*MKValueFunc)(MKVariable *var, MKInterp *interp, int is_not,
			   int test);

struct _mk_variable {
  Node node;

  char *name;

  SBuf value;			/* value */

  MKValueFunc vfunc;		/* if set, used instaed of value */
  void *data;			/* additional user data */

  unsigned int temp_level : 3;	/* is it a temporarly variable? */
  unsigned int read_only  : 1;	/* is this variable read only? */
};

/* MKLoop ****************************************************************** */

typedef struct _mk_loop MKLoop;

typedef void (*MKLoopFunc)(MKLoop *ldata, MKInterp *interp, NodeList *stmts);

struct _mk_loop {
  Node node;

  char *name;
  MKLoopFunc lfunc;		/* function to handle the loop. */
  void *data;			/* additional user data */
};

/* MKInterp **************************************************************** */

struct _mk_interpreter {
  NodeList variables;		/* List of MKVariable */
  NodeList loops;		/* List of MKLoop */

  FILE *out;			/* output file */

  SBuf buf;			/* string buffert */

  const char *userhome;		/* home directory of user */
  const char *stdpath;		/* base path if no path set */
};

/* Externs ***************************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

#define mkinterp_init(interp) \
  (interp)->variables.head = (interp)->variables.tail = NULL; \
  (interp)->loops.head = (interp)->loops.tail = NULL; (interp)->out = NULL; \
  sbuf_init(&(interp)->buf)

extern int mkloop_set(MKInterp *interp, const char *name, MKLoopFunc lfunc, 
		      void *data);

extern int mkvar_set(MKInterp *interp, const char *name, const char *value, 
		     MKValueFunc vfunc, void *data, int readonly, 
		     int temp_level, int override);

extern MKVariable *mkvar_find(MKInterp *interp, const char *name);
extern int mkvar_unset(MKInterp *interp, const char *name, int override);
extern void mkvar_temp_unset(MKInterp *interp, int temp_level);

extern void mkvar_fl_store(MKInterp *interp, int *first, int *last);
extern void mkvar_fl_restore(MKInterp *interp, int first, int last);
extern void mkvar_fl_set(MKInterp *interp, NodeList *list, Node *node);

extern int mkinterp_exec(const char *function, enum MKClass fclass, 
			 NodeList *funcs, MKInterp *interp);

extern void mkinterp_exec_stmts(NodeList *stmts, MKInterp *interp);
extern void mkinterp_exec_stmt(MKStmt *stmt, MKInterp *interp);

extern void mkinterp_clear(MKInterp *interp);

#ifdef __cplusplus
}
#endif

#endif /* _MKINTERP_H */
