/* HICKMK Lib 1.2.1 - HICK Make Utility
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _MKLANG_H		/* Load routines for MK Language */
#define _MKLANG_H

#include <sys/types.h>
#include <sys/stat.h>

#include "nodes.h"

enum MKToken {
  MKT_DIR      = 300,		/* 2: mode, path                      */
  MKT_FILE     = 301,		/* 2: mode, filename                  */
  MKT_FOREACH  = 302,		/* 2: loop ID, stmt                   */
  MKT_READ     = 303,		/* 1: list of files (stored in stmt1) */
  MKT_WRITE    = 304,		/* 1: string                          */
  MKT_IF       = 305,		/* 3: expression, stmt1 [ , stmt2 ]   */
  MKT_UNSET    = 306,		/* 1: variable                        */
  MKT_EXPR     = 307		/* 2: expression                      */
};

enum MKOperator {
  MKO_ID        = 0,
  MKO_EQUAL     = 1,
  MKO_NOT_EQUAL = 2,
  MKO_SET       = 3
};

enum MKClass {
  MKC_MENU    = 0,
  MKC_SCRIPT  = 1,
  MKC_HTML    = 2
};

enum MKLang_Type { MKTLang, MKTInterp, MKTParse };
enum MKLang_Err { MKError, MKWarning, MKFatal };

/* MKExpr ****************************************************************** */

typedef struct {
  Node node;			/* left (next) and right (prev) */

  unsigned int op;		/* operator */

  char *value;			/* value */
} MKExpr;

/* MKStmt ****************************************************************** */

typedef struct {
  Node node;

  unsigned int token;		/* Using the MKToken */

  char *value;			/* value to functions */
  NodeList stmt1;		/* statement list */
  NodeList stmt2;		/* if else statment */

  union {
    MKExpr *expr;		/* expression */
    mode_t mode;		/* for DIR and FILE */
  } u;
} MKStmt;

/* MKFunction ************************************************************** */

typedef struct {
  MKStmt stmt;			/* first statement */

  char *name;			/* function name */
  enum MKClass fclass;		/* belongs to class */
} MKFunction;

/* Externs ***************************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

extern int mklang_functions_load(const char *filename, NodeList *list);
extern void mklang_functions_free(NodeList *list);

extern void mklang_error(enum MKLang_Type type, enum MKLang_Err err, 
			 const char *fmt, ...);

#ifdef __cplusplus
}
#endif

#endif /* _MKLANG_H */
