/* HICKMK Lib 1.2.1 - HICK Make Utility
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _SBUF_H			/* String buffert */
#define _SBUF_H

#include <sys/types.h>

#define SBUF_REALLOC 32

/* SBuf ******************************************************************** */

typedef struct {
  char *buf;
  size_t siz;
  size_t alloc;
} SBuf;

/* Externs ***************************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

#define sbuf_init(sbuf) \
   (sbuf)->siz = (sbuf)->alloc = 0; (sbuf)->buf = 0;

#define sbuf_put(c, sbuf) \
   if((((sbuf)->siz + 1) > (sbuf)->alloc)) { \
     (sbuf)->alloc += SBUF_REALLOC; \
     if((sbuf)->alloc == SBUF_REALLOC) { \
       (sbuf)->buf = MallocArr(char, (sbuf)->alloc); \
     } else { \
       (sbuf)->buf = ReMallocArr((sbuf)->buf, char, (sbuf)->alloc); \
     } \
   } \
   (sbuf)->buf[(sbuf)->siz++] = c


#define sbuf_clear(sbuf) \
   if((sbuf)->buf) { \
     FreeMem((sbuf)->buf); \
   } \
   sbuf_init(sbuf);


#define sbuf_move(from, to) \
   (to)->buf = (from)->buf; (to)->siz = (from)->siz; \
   (to)->alloc = (from)->alloc; sbuf_init(from)


extern void sbuf_puts(const char *s, SBuf *to);
extern void sbuf_putsb(SBuf *from, SBuf *to);

#ifdef __cplusplus
}
#endif

#endif /* _SBUF_H */
