/* HickSH Lib 1.1.1 - Shared routines between hick parts.
** Copyright (C) 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _HICKSH_H
#define _HICKSH_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# ifdef HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# ifdef HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# ifdef HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif

#include <memd.h>

typedef struct {
  char **arr;
  size_t siz;
  size_t alloc;
} ArrayInfo;


#define FNAME_LEN    1024
#define ARRBUF_SIZE  5

#define IOW_OK    0
#define IOW_FAIL  1

#ifdef STAT_MACROS_BROKEN
#undef S_ISDIR
#undef S_ISREG

#define S_ISDIR(mode) (((mode) & 0xF000) == 0x4000)
#define S_ISREG(mode) (((mode) & 0xF000) == 0x8000)
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* strr.c */

#define hsh_array_init(arri) (arri)->arr = NULL; (arri)->siz=(arri)->alloc = 0

extern int hsh_array_add(char *name, int copy, ArrayInfo *arri);
extern int hsh_array_find(const char *name, ArrayInfo *arri);
extern int hsh_array_match(ArrayInfo *arri1, ArrayInfo *arri2);
extern void hsh_array_remove(ArrayInfo *arri);

extern char *hsh_create_str(char *buf, size_t maxlen, ...);

/* iowrap.c */

extern int hsh_mkdir(const char *path, mode_t mode, FILE *logfile);
extern int hsh_rmdir(const char *path, FILE *logfile);

extern int hsh_unlink(const char *path, FILE *logfile);
extern int hsh_symlink(const char *name1, const char *name2, FILE *logfile);

extern int hsh_rename(const char *oname, const char *nname, FILE *logfile);

extern int hsh_chdir(const char *path, FILE *logfile);
extern char *hsh_getcwd(char *buf, size_t size, FILE *logfile);

extern int hsh_stat(const char *path, struct stat *sbuf, FILE *logfile);
extern int hsh_lstat(const char *path, struct stat *sbuf, FILE *logfile);

extern int hsh_chmod(const char *path, mode_t mode, FILE *logfile);

extern DIR *hsh_opendir(const char *path, FILE *logfile);

extern int hsh_path_create(const char *path, FILE *logfile);

#ifdef __cplusplus
}
#endif

#endif /* _HICKSH_H */
