/* HickSH Lib 1.1.1 - Shared routines between hick parts.
** Copyright (C) 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>

#include "memd.h"
#include "hstring.h"

#define HSTRING_REALLOC 16

static char *hstring_realloc(HString *s, size_t alloc);


void
hstring_add_char(HString *s, char c)
{
  if((s->len + 2) > s->alloc) {
    s->data = hstring_realloc(s, HSTRING_REALLOC);
  }
    
  s->data[s->len++] = c;
}


void
hstring_add_string(HString *s, char *str, size_t len)
{
  size_t n;

  if((s->len + len + 1) > s->alloc) {
    s->data = hstring_realloc(s, len + HSTRING_REALLOC);
  }

  for(n = 0; n < len; n++) {
    s->data[s->len++] = str[n];
  }
}


void
hstring_eos(HString *s)
{
  if(s->len >= s->alloc) {
    s->data = hstring_realloc(s, HSTRING_REALLOC);
  }

  s->data[s->len] = (char)0;
}


void
hstring_clear(HString *s)
{
  s->len = 0;
}


void
hstring_free(HString *s)
{
  if(s->data) {
    FreeMem(s->data);
  }

  hstring_init(s);
}


static char *
hstring_realloc(HString *s, size_t alloc)
{
  s->alloc += alloc;

  if(s->data) {
    s->data = ReMallocArr(s->data, char, s->alloc);
  } else {
    s->data = MallocArr(char, s->alloc);
  }

  if(!s->data) {
    fprintf(stderr, "Out of memory -- exiting.\n");
    exit(1);
  }

  return s->data;
}
