/* Memory Debugger 1.2.1 - Runtime memory debugger.
** Copyright (C) 95, 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _MEMD_H
#define _MEMD_H

#include <stdlib.h>

#ifndef MEMDEBUG

#define MallocMem(t)             (t *)malloc(sizeof(t))
#define MallocArr(t, n)          (t *)malloc((sizeof(t) * (n)))

#define ReMallocMem(ptr, t)      (t *)realloc((ptr), sizeof(t))
#define ReMallocArr(ptr, t, n)   (t *)realloc((ptr), (sizeof(t) * (n)))

#define FreeMem(ptr)             free((void *)ptr)

#define MemDebugLine(ptr, t)     (t *)(ptr)
#define MemDebugHide(ptr, t)     (t *)(ptr)
#define MemDebugShow(ptr, t)     (t *)(ptr)

#define ShowMem
#define ClearMem

#else  /* MEMDEBUG */

#define MallocMem(t) \
       (t *)_memd_alloc(sizeof(t), __FILE__, __LINE__)

#define MallocArr(t, n) \
       (t *)_memd_alloc((size_t)(sizeof(t) * (n)), __FILE__, __LINE__)

#define ReMallocMem(ptr, t) \
       (t *)_memd_realloc((void *)(ptr), sizeof(t), __FILE__, __LINE__)

#define ReMallocArr(ptr, t, n) \
       (t *)_memd_realloc((void *)(ptr), (sizeof(t) * (n)), __FILE__, __LINE__)

#define FreeMem(ptr)  \
       _memd_free((void *)ptr, __FILE__, __LINE__)

#define MemDebugLine(ptr, t) \
       (t *)_memd_setline(ptr, __FILE__, __LINE__)

#define MemDebugHide(ptr, t) \
       (t *)_memd_setshow(ptr, 0, __FILE__, __LINE__)

#define MemDebugShow(ptr, t) \
       (t *)_memd_setshow(ptr, 1, __FILE__, __LINE__)

#define ShowMem        _memd_showmem(__FILE__, __LINE__)
#define ClearMem       _memd_clearmem(__FILE__, __LINE__)

#ifdef __cplusplus
extern "C" {
#endif

extern void *_memd_alloc(size_t size, const char *file, long line);
extern void *_memd_realloc(void *ptr, size_t size, const char *file,
			   long line);
extern void _memd_free(void *ptr, const char *file, long line);

extern void *_memd_setline(void *ptr, const char *file, long line);
extern void *_memd_setshow(void *ptr, char value, const char *file, long line);

extern void _memd_showmem(const char *file, long line);
extern void _memd_clearmem(const char *file, long line);

#ifdef __cplusplus
}
#endif

#endif /* MEMDEBUG */

#endif /* _MEMD_H */
