/* HickSH Lib 1.1.1 - HICK Make Utility
** Copyright (C) 98, 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* DEFINES:
**
** NODE_PROTECTED : Extra tests to make sure we don't do something wrong.
**   NODE_DEBUG   : Print error messages, used with NODE_PROTECTED.
*/

#ifndef _NODES_H
#define _NODES_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "memd.h"

#ifdef NODE_PROTECTED
#define NODE_IS_NOT_NULL(func, ptr1, ptr2) \
    if(!(ptr1) || !(ptr2)) { \
      NODE_DEBUG_PTRS(func, ptr1, ptr2) \
      return NULL; \
    }

#define NODE_PTR_NOT_EQUAL(func, ptr1, ptr2) \
    if((ptr1) != (ptr2)) { \
      NODE_DEBUG_STRS(func, ptr1, ptr2); \
      return NULL; \
    }
#else  /* not NODE_PROTECTED */
#define NODE_IS_NOT_NULL(func, ptr1, ptr2)
#define NODE_PTR_NOT_EQUAL(func, ptr1, ptr2)
#endif /* not NODE_PROTECTED */

#ifdef NODE_DEBUG
#define NODE_DEBUG_PTRS(func, ptr1, ptr2) \
    if(!(ptr1)) { \
      fprintf(stderr, "ERROR: %s: passing NULL on %s.\n", (func), #ptr1); \
    } \
    if(!(ptr2)) { \
      fprintf(stderr, "ERROR: %s: passing NULL on %s.\n", (func), #ptr2); \
    }
#define NODE_DEBUG_STRS(func, ptr1, ptr2) \
    fprintf(stderr, "ERROR: %s: %s is not equal to %s.\n", (func),#ptr1,#ptr2);
#else  /* not NODE_DEBUG */
#define NODE_DEBUG_STRS(func, ptr1, ptr2)
#define NODE_DEBUG_PTRS(func, ptr1, ptr2)
#endif /* NODE_DEBUG */

/* Node ******************************************************************** */

typedef struct _node {
  struct _node *next;
  struct _node *prev;
} Node;

/* NodeName **************************************************************** */

typedef struct {
  Node node;
  char *name;
} NodeName;

/* NodeNum ***************************************************************** */

typedef struct {
  Node node;
  unsigned int num;
} NodeNum;

/* NodeList **************************************************************** */

typedef struct {
  Node *head;
  Node *tail;
} NodeList;

/* Externs ***************************************************************** */

#ifdef __cplusplus
extern "C" {
#endif

#define hsh_nodelist_init(l)  (l)->head = (l)->tail = NULL

extern Node *hsh_node_prepend(Node *node, NodeList *to);
extern Node *hsh_node_append(Node *node, NodeList *to);
extern Node *hsh_node_remove(Node *node, NodeList *from);
extern Node *hsh_node_insert_after(Node *node, Node *after, NodeList *to);

extern Node *hsh_node_name_insert(NodeName *nname, NodeList *to, int rev);
extern Node *hsh_node_num_insert(NodeNum *nnum, NodeList *to, int rev);

extern NodeName *hsh_node_name_find(const char *name, NodeList *list);

#ifdef __cplusplus
}
#endif

#endif /* _NODES_H */
