/* HickSH Lib 1.1.1 - Shared routines between hick parts.
** Copyright (C) 99, 2000 Daniel Bergstroem <daniel@sdf.se>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdarg.h>
#include <strings.h>

#include "memd.h"
#include "hicksh.h"


char *
hsh_create_str(char *buf, size_t maxlen, ...)
{
  char *p;
  size_t n = 0;
  va_list pvar;

  va_start(pvar, maxlen);

  while((p = va_arg(pvar, char *)) && (n < maxlen)) {
    while(*p && (n < maxlen)) {
      buf[n++] = *p++;
    }
  }

  buf[n] = (char)0;

  va_end(pvar);
  return buf;
}


int
hsh_array_add(char *name, int copy, ArrayInfo *arri)
{
  int pos;

  if(name) {
    if((pos = hsh_array_find(name, arri))) {
      if(!copy) {
	FreeMem(name);
      }
      return pos;
    }

    if((arri->siz + 1) > arri->alloc) {
      if(arri->arr) {
	arri->arr = ReMallocArr(arri->arr, char *, arri->alloc + ARRBUF_SIZE);
      } else {
	arri->arr = MallocArr(char *, ARRBUF_SIZE);
      }

      if(!arri->arr) {
	return 0;
      }

      arri->alloc += ARRBUF_SIZE;
    }

    if(copy) {
      if((arri->arr[arri->siz] = MallocArr(char, strlen(name) + 1))) {
	strcpy(arri->arr[arri->siz++], name);
	return arri->siz;
      }
    } else {
      arri->arr[arri->siz++] = name;
    }
  }

  return 0;
}


void
hsh_array_remove(ArrayInfo *arri)
{
  size_t n;

  if(arri->arr) {
    for(n = 0; n < arri->siz; n++) {
      FreeMem(arri->arr[n]);
    }

    FreeMem(arri->arr);
    hsh_array_init(arri);
  }
}


int
hsh_array_find(const char *name, ArrayInfo *arri)
{
  size_t n;

  for(n = 0; n < arri->siz; n++) {
    if(!strcasecmp(name, arri->arr[n])) {
      return (n + 1);
    }
  }

  return 0;
}


int
hsh_array_match(ArrayInfo *arri1, ArrayInfo *arri2)
{
  size_t n;

  if(arri1 != arri2) {
    if(arri1->siz != arri2->siz) {
      return 0;
    }

    for(n = 0; n < arri1->siz; n++) {
      if(!hsh_array_find(arri1->arr[n], arri2)) {
	return 0;
      }
    }
  }

  return 1;
}
